/* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */

/* This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * (C) Copyright 2009 Novell, Inc.
 */

#include <NetworkManager.h>
#include <nm-connection.h>
#include <nm-setting-connection.h>
#include "nmn-ethernet-item.h"

G_DEFINE_TYPE (NmnEthernetItem, nmn_ethernet_item, NMN_TYPE_NETWORK_ITEM)

#define GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), NMN_TYPE_ETHERNET_ITEM, NmnEthernetItemPrivate))

typedef struct {
    gulong state_changed_id;
    gboolean disposed;
} NmnEthernetItemPrivate;

static void
state_changed (NMDevice *device,
               NMDeviceState new_state,
               NMDeviceState old_state,
               NMDeviceStateReason reason,
               gpointer user_data)
{
    const char *icon;

    switch (new_state) {
    case NM_DEVICE_STATE_ACTIVATED:
        icon = "nm-device-wired";
        break;
    default:
        icon = "nm-no-connection";
        break;
    }

    nmn_network_item_set_icon (NMN_NETWORK_ITEM (user_data), icon);
}

GtkWidget *
nmn_ethernet_item_new (NmnNMData *nm_data,
                       NMDeviceEthernet *device)
{
    g_return_val_if_fail (NMN_IS_NM_DATA (nm_data), NULL);
    g_return_val_if_fail (NM_IS_DEVICE_ETHERNET (device), NULL);

    return GTK_WIDGET (g_object_new (NMN_TYPE_ETHERNET_ITEM,
                                     NMN_NETWORK_ITEM_NM_DATA, nm_data,
                                     NMN_NETWORK_ITEM_DEVICE, device,
                                     NULL));
}

static void
update_cb (NMSettingsConnectionInterface *connection,
	   GError *error,
	   gpointer data)
{
	NMSettingConnection *s_con;
	NmnNetworkItem *item = data;

	if (error != NULL) {
		g_warning ("Updating auto-connect for ethernet failed: %s", error->message);
		return;
	}
	s_con = NM_SETTING_CONNECTION (connection);
	if (nm_setting_connection_get_autoconnect (s_con) == TRUE) {
		NMN_NETWORK_ITEM_CLASS (nmn_ethernet_item_parent_class)->connect (item);
	} else {
		NMN_NETWORK_ITEM_CLASS (nmn_ethernet_item_parent_class)->disconnect (item);
	}
}

static gboolean
update_autoconnect (NmnNetworkItem *item, gboolean connect_automatically)
{
    NMSettingsConnectionInterface *connection;
    NMSettingConnection *s_con;
    GError *error = NULL;

    connection = nmn_network_item_get_connection (item);
    s_con = NM_SETTING_CONNECTION (nm_connection_get_setting (NM_CONNECTION (connection), NM_TYPE_SETTING_CONNECTION));

    if (nm_setting_connection_get_autoconnect (s_con) == connect_automatically)
        return FALSE;

    g_object_set (s_con, NM_SETTING_CONNECTION_AUTOCONNECT, connect_automatically, NULL);
    nm_settings_connection_interface_update (NM_SETTINGS_CONNECTION_INTERFACE (connection),
                                             update_cb,
                                             item);

    return TRUE;
}

static void
connect (NmnNetworkItem *item)
{
    update_autoconnect (NMN_NETWORK_ITEM (item), TRUE);
}

static void
disconnect (NmnNetworkItem *item)
{
    /* Turn off autoconnect, otherwise it would reconnect right back. */
    update_autoconnect (item, FALSE);
}

static guint
get_priority (NmnNetworkItem *item)
{
    return NMN_NETWORK_ITEM_CLASS (nmn_ethernet_item_parent_class)->get_priority (item) + 50;
}

static void
nmn_ethernet_item_init (NmnEthernetItem *item)
{
}

static GObject*
constructor (GType type,
             guint n_construct_params,
             GObjectConstructParam *construct_params)
{
    GObject *object;
    NmnEthernetItemPrivate *priv;
    NMDevice *device;

    object = G_OBJECT_CLASS (nmn_ethernet_item_parent_class)->constructor
        (type, n_construct_params, construct_params);

    if (!object)
        return NULL;

    priv = GET_PRIVATE (object);
    device = nmn_network_item_get_device (NMN_NETWORK_ITEM (object));
    priv->state_changed_id = g_signal_connect (device, "state-changed", G_CALLBACK (state_changed), object);

    state_changed (device, nm_device_get_state (device), 
                   NM_DEVICE_STATE_UNKNOWN, NM_DEVICE_STATE_REASON_NONE, object);

    return object;
}

static void
dispose (GObject *object)
{
    NmnEthernetItemPrivate *priv = GET_PRIVATE (object);
    NMDevice *device;

    if (priv->disposed)
        return;

    device = nmn_network_item_get_device (NMN_NETWORK_ITEM (object));
    g_signal_handler_disconnect (device, priv->state_changed_id);

    priv->disposed = TRUE;

    G_OBJECT_CLASS (nmn_ethernet_item_parent_class)->dispose (object);
}

static void
nmn_ethernet_item_class_init (NmnEthernetItemClass *class)
{
    GObjectClass *object_class = G_OBJECT_CLASS (class);
    NmnNetworkItemClass *network_item_class = NMN_NETWORK_ITEM_CLASS (class);

    g_type_class_add_private (object_class, sizeof (NmnEthernetItemPrivate));

    object_class->constructor = constructor;
    object_class->dispose = dispose;

    network_item_class->connect = connect;
    network_item_class->disconnect = disconnect;
    network_item_class->get_priority = get_priority;
}
